<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Teknisa\MAN\Utilities\Utilities;


if (!defined('PATH_NFCEPHP')) {
    define('PATH_NFCEPHP', Utilities::nfcePath());
}

class ConfigAmbientNFCE{

	protected $getParamService;

	public function __construct(\Service\ConfigAmbientNFCE $configAmbientNFCEService) {
		$this->configAmbientNFCEService = $configAmbientNFCEService;
	}

	public function getConfigData(Request\Row $request, Response $response){	
		try{
			$row = $request->getRow();
			$cdfilial = $row['CDFILIAL'];
			$nrinsjurfili = $row['NRINSJURFILI'];
			$nrorg = $row['NRORG'];
			
			$filialData = $this->configAmbientNFCEService->getFilialData($cdfilial, $nrorg);
			$placeData = $this->configAmbientNFCEService->getPlaceData($cdfilial, $nrorg);
			$paravend = $this->configAmbientNFCEService->getParavend($cdfilial, $nrorg);
			$fileData = $this->configAmbientNFCEService->getFileData($cdfilial, $nrorg, $nrinsjurfili);

            if(count($fileData) > 0){
            	self::mergeArrays($fileData);
            	self::deconcatPath($fileData);
            }else{
            	self::fillPathFields($fileData);
            	self::fillDefaultFields($fileData);
            }
			if(count($filialData) > 0){
				$fileData['cnpj'] = isset($filialData[0]['NRINSJURFILI']) ? $filialData[0]['NRINSJURFILI'] : "";
				$fileData['razaosocial'] = isset($filialData[0]['NMRAZSOCFILI']) ? $filialData[0]['NMRAZSOCFILI'] : "";
			}
			if(count($placeData) > 0){
				$fileData['siglaUF'] = isset($placeData[0]['SGESTADO']) ? $placeData[0]['SGESTADO'] : "";
			}
			if(count($paravend) > 0){		
				if ($paravend[0]['IDAMBTRABNFCE'] != null){
					$fileData['tpAmb'] = $paravend[0]['IDAMBTRABNFCE'];
				}
				else {
					$fileData['tpAmb'] = isset($fileData['tpAmb']) ? $fileData['tpAmb'] : "2"; // Homologação
				}
				if ($paravend[0]['DSDIRARQLOGNFCE'] != null){
					$fileData['pathNFeFiles'] = $paravend[0]['DSDIRARQLOGNFCE'];
				}
				else {
					$fileData['pathNFeFiles'] = isset($fileData['pathNFeFiles']) ? $fileData['pathNFeFiles'] : "";
				}

				if ($paravend[0]['CDCODSCONSHOMO'] != null){
					$fileData['tokenNFCe'] = $paravend[0]['CDCODSCONSHOMO'];
				}
				else {
					$fileData['tokenNFCe'] = isset($fileData['tokenNFCe']) ? $fileData['tokenNFCe'] : "";	
				}

				if ($paravend[0]['CDIDTOKENHOMO'] != null){
					$fileData['tokenNFCeId'] = $paravend[0]['CDIDTOKENHOMO'];
				}
				else {
					$fileData['tokenNFCeId'] = isset($fileData['tokenNFCeId']) ? $fileData['tokenNFCeId'] : "";	
				}				
			}
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("configData", $fileData));

		}catch(Exception $e){
			var_dump($e->getMessage());
		}
	}

	public function saveConfigData(Request\Row $request, Response $response){	
		try{
			$row = $request->getRow();
			$nrorg = $row['NRORG'];
			$nrinsjurfili = $row['cnpj'];
			$row['pathCertsFiles'] = PATH_NFCEPHP.DIRECTORY_SEPARATOR.$row['pathCertsFiles'];
			$message = $this->configAmbientNFCEService->saveFileData($row, $nrinsjurfili, $nrorg);

			$response->addMessage(new Message($message));
		}catch(Exception $e){
			var_dump($e->getMessage());
		}
	}

	public function fillPathFields(&$fileData){
		$fileData['pathXmlUrlFileNFe'] = 'nfe_ws3_mod65.xml';
		$fileData['pathXmlUrlFileCTe'] = 'cte_ws2.xml';
		$fileData['pathXmlUrlFileMDFe'] = 'mdfe_ws1.xml';
		$fileData['pathCertsFiles'] = 'certs'.DIRECTORY_SEPARATOR;
		$fileData['schemesNFe'] = 'PL_008i1';
		$fileData['schemesCTe'] = 'PL_CTe_200';
		$fileData['schemesMDFe'] = 'PL_MDFe_100';
		$fileData['siteUrl'] = str_replace('index.php', '', 'http://'.$_SERVER["SERVER_NAME"].$_SERVER["SCRIPT_NAME"]);
	}

	public function fillDefaultFields(&$fileData){
		$fileData['tpAmb'] = "2";
		$fileData['siglaUF'] = "SP";
		$fileData['format'] = "L";
		$fileData['paper'] = "A4";
		$fileData['southpaw'] = true;
		$fileData['logoPosition'] = "L";
		$fileData['font'] = "Times";
		$fileData['mailAuth'] = true;
		$fileData['mailProtocol'] = "none";
	}

	public function mergeArrays(&$fileData){
		$index = array('aDocFormat', 'aMailConf', 'aProxyConf');
		foreach ($index as $id) {
			foreach ($fileData[$id] as $key => $value) {
				$fileData[$key] = $value;
			}
			unset($fileData[$id]);
		}
	}

	public function deconcatPath(&$fileData){
		$fileData['pathCertsFiles'] = explode(PATH_NFCEPHP.DIRECTORY_SEPARATOR, $fileData['pathCertsFiles'])[1];
	}

	public function searchToFile(Request\Row $request, Response $response){
		$row = $request->getRow();
		$fileOnPath = false;
		$searchFile = "";
		$arrayToResponse = Array();
		$messageDefault = "Não encontrado no diretório: ";
			
		if ($row['widgetName'] == 'emitente'){
			if (is_dir(PATH_NFCEPHP.DIRECTORY_SEPARATOR.$row['pathCertsFiles'])) {
				$files = preg_grep('/^([^.])/', scandir(PATH_NFCEPHP.DIRECTORY_SEPARATOR.$row['pathCertsFiles']));
				foreach ($files as $file) {
					if ($file == $row['certPfxName']){
						$fileOnPath = true;
					}
				}
				$searchFile = $row['certPfxName'];
			} 
			else {
				$fileOnPath = false;
				$messageDefault = "Path dos Certificados não encontrado!";
				$searchFile = "";
			}

		}
		elseif ($row['widgetName'] == 'webservices'){
			$filesToSearch = ['pathXmlUrlFileNFe', 'pathXmlUrlFileCTe', 'pathXmlUrlFileMDFe'];
			$files = preg_grep('/^([^.])/', scandir('C:\workfolder\management\backend\vendor\nfephp-org\nfephp\config'.DIRECTORY_SEPARATOR));
			foreach ($filesToSearch as $campo) {
				$nameFile = $row[$campo];
				$found = false;
				foreach ($files as $file) {
					if (!(is_dir('C:\workfolder\management\backend\vendor\nfephp-org\nfephp\config'.DIRECTORY_SEPARATOR.$file)) && $file == $nameFile){
						$found = true;
					}
				}				
				if (!$found){
					if ($searchFile == ""){
						$searchFile = $nameFile;
					}
					else {
						$searchFile = $searchFile.", ".$nameFile;
					}
				}
			}
			if ($searchFile == ""){
				$fileOnPath = true;				
			}
		}

		if ($fileOnPath){
			$arrayToResponse['STATUS'] = true;
		}
		else {
			$arrayToResponse['STATUS'] = false;
			$arrayToResponse['NOTIFICATION'] = $messageDefault.$searchFile;
		}

		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('searchToFile', array($arrayToResponse)));
	}

}