<?php
namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities\Query;

class FamiliadeProdutoService {

    /**
     * @var 
     */
    protected $connection;
    protected $entityManager;
    public function __construct(Connection $connection, $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }

    public function ValidProduct($conditions){
    	$params = array(
			':CDFILIAL' => $conditions['CDFILIAL'],
			':CDPRODUTO' 	=> $conditions['CDPRODUTO'],
			':NRORG'    => $conditions['NRORG']
		);
    	return $this->connection->fetchAll(self::VERIFY_PROD_ON_FAMILIA, $params);
    }
    
        

    
    public function selectProductAutoCad($conditions){
        try {	
            $params = array(
    			':NRORG' => $conditions['NRORG']
    		);
    		$CDNVProduto = $this->entityManager->getConnection()->fetchAll(Query::GET_MAXCDNVPRODUTO, $params);
    		unset($params);
            $params = array(
    			':CDPRODFIN'     => $conditions['CDARVPRODFINAL'],
    			':CDFAMILISALD'  => $conditions['CDFAMILISALD'],
    			':CDFILIAL'      => $conditions['CDFILIAL'],
    			':QTCONSDIAFAM'  => $conditions['QTCONSDIAFAM'],
    			':CDPRODINI'     => $conditions['CDARVPRODINI'],
    			':CDNVPRODUTO'   => $CDNVProduto[0]['CDNVPRODUTO'],
    			':NRORG'         => $conditions['NRORG']
    
    		);
        	return $this->connection->fetchAll(self::SELECT_PROD_AUTO_CAD, $params);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insertProducts($produto){
        try{
        $params = array(
            ':CDFILIAL'         => $produto['CDFILIAL'],
            ':CDFAMILISALD'     => $produto['CDFAMILISALD'],
            ':CDPRODUTO'        => $produto['CDPRODUTO'],  
            ':QTCONSDIAFAM'     => $produto['QTCONSDIAFAM']
            
        );
        return $this->entityManager->getConnection()->executeQuery(self::INSERT_PRODUTOS, $params);
        } catch(Exception $e){
            throw $e;
        }
    }
    
    public function ValidProductCadAuto($conditions){
        try {	
            $params = array(
    			':NRORG' => $conditions['NRORG']
    		);
    		$CDNVProduto = $this->entityManager->getConnection()->fetchAll(Query::GET_MAXCDNVPRODUTO, $params);
    		unset($params);
            $params = array(
    			':CDPRODFIN'     => $conditions['CDARVPRODFINAL'],
    			':CDFAMILISALD'  => $conditions['CDFAMILISALD'],
    			':CDFILIAL'      => $conditions['CDFILIAL'],
    			':QTCONSDIAFAM'  => $conditions['QTCONSDIAFAM'],
    			':CDPRODINI'     => $conditions['CDARVPRODINI'],
    			':CDNVPRODUTO'   => $CDNVProduto[0]['CDNVPRODUTO'],
    			':NRORG'         => $conditions['NRORG']
    
    		);
        	return $this->connection->fetchAll(self::VALID_PROD_AUTO_CAD, $params);
        } catch (Exception $e) {
            throw $e;
        }
    }

    const VALID_PROD_AUTO_CAD = "
            SELECT 1
                    FROM FAMSALDOPROD FAM, PRODUTO PRO
                WHERE (FAM.CDFILIAL = :CDFILIAL)
                    AND (PRO.CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)
                    AND (PRO.CDPRODUTO = FAM.CDPRODUTO)
    ";

    const SELECT_PROD_AUTO_CAD = "
            SELECT DISTINCT PR.CDPRODUTO,
              PR.CDARVPROD,
              PR.CDNVPRODUTO,  
              PR.NMPRODUTO,
              PR.NRORG,
              :CDFILIAL AS CDFILIAL,
              :CDFAMILISALD  AS CDFAMILISALD,
              :QTCONSDIAFAM AS QTCONSDIAFAM
            FROM PRODUTO PR
            LEFT JOIN PRODFILI PF
             ON PR.CDPRODUTO              = PF.CDPRODUTO
              AND PR.NRORG                 = PF.NRORG
              AND PF.CDFILIAL              = :CDFILIAL
            WHERE (PR.CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)
              AND PR.CDNVPRODUTO         = :CDNVPRODUTO
              AND (PR.CDARVPROD NOT IN (SELECT PRO.CDPRODUTO
                              FROM FAMSALDOPROD FAM,
                              PRODUTO PRO
                            WHERE (FAM.CDFILIAL = :CDFILIAL)
                            AND(PRO.CDARVPROD BETWEEN :CDPRODINI AND :CDPRODFIN)
                            AND(PRO.CDPRODUTO = FAM.CDPRODUTO)))
              AND PR.NRORG                 = :NRORG
              AND NVL(PF.IDPRODATIV,'A')   = 'A'
              AND NVL(PR.IDPRODATIVPR,'S') = 'S'
            ORDER BY PR.CDARVPROD      
    ";
   
    const VERIFY_PROD_ON_FAMILIA = "
    	SELECT FP.CDFILIAL, FP.CDFAMILISALD, FP.CDPRODUTO, FS.NMFAMILISALD
		  FROM FAMSALDOPROD FP, FAMILIASALDO FS
		 WHERE (FP.CDFILIAL = :CDFILIAL)
		   AND (FP.CDPRODUTO = :CDPRODUTO)
		   AND (FP.NRORG = :NRORG)
		   AND (FP.CDFAMILISALD = FS.CDFAMILISALD)
    ";
    
    const INSERT_PRODUTOS = "
       INSERT INTO FAMSALDOPROD (CDFILIAL, CDFAMILISALD, CDPRODUTO, QTCONSDIAFAM)
        VALUES (:CDFILIAL, :CDFAMILISALD, :CDPRODUTO, :QTCONSDIAFAM)
    ";
}